import React, { useState, useEffect, useRef } from 'react';
import { Car, Volume2, Volume, AlertTriangle, Bluetooth, BluetoothOff } from 'lucide-react';

function App() {
  const [connected, setConnected] = useState(false);
  const [leftDistance, setLeftDistance] = useState<number | null>(null);
  const [rightDistance, setRightDistance] = useState<number | null>(null);
  const [soundEnabled, setSoundEnabled] = useState(true);
  const [status, setStatus] = useState('Waiting for connection');
  const [instruction, setInstruction] = useState('Connect to Arduino to start monitoring');
  const [alertLevel, setAlertLevel] = useState<'safe' | 'warning' | 'danger'>('safe');
  
  const port = useRef<any>(null);
  const reader = useRef<any>(null);
  const audioContext = useRef<AudioContext | null>(null);
  const oscillator = useRef<OscillatorNode | null>(null);
  const gainNode = useRef<GainNode | null>(null);
  const beepIntervalRef = useRef<number | null>(null);

  // Initialize audio context
  useEffect(() => {
    if (typeof window !== 'undefined') {
      audioContext.current = new (window.AudioContext || (window as any).webkitAudioContext)();
      gainNode.current = audioContext.current.createGain();
      gainNode.current.gain.value = 0;
      gainNode.current.connect(audioContext.current.destination);
    }

    return () => {
      if (oscillator.current) {
        oscillator.current.stop();
        oscillator.current.disconnect();
      }
      if (beepIntervalRef.current) {
        clearInterval(beepIntervalRef.current);
      }
    };
  }, []);

  // Function to play beep sound with frequency based on distance
  const playBeep = (minDistance: number) => {
    if (!soundEnabled || !audioContext.current || !gainNode.current) return;
    
    // Clear any existing interval
    if (beepIntervalRef.current) {
      clearInterval(beepIntervalRef.current);
      beepIntervalRef.current = null;
    }
    
    if (minDistance >= 100) {
      return; // No sound for safe distances
    }
    
    // Calculate interval based on distance (closer = faster beeps)
    const beepInterval = Math.max(100, minDistance * 10);
    
    const createBeep = () => {
      if (oscillator.current) {
        oscillator.current.stop();
        oscillator.current.disconnect();
      }
      
      oscillator.current = audioContext.current!.createOscillator();
      oscillator.current.type = 'sine';
      
      // Adjust frequency based on distance (closer = higher pitch)
      oscillator.current.frequency.value = 440 + (100 - minDistance) * 5;
      
      oscillator.current.connect(gainNode.current!);
      oscillator.current.start();
      
      gainNode.current!.gain.value = 0.2;
      
      setTimeout(() => {
        if (gainNode.current) gainNode.current.gain.value = 0;
      }, 100);
    };
    
    createBeep();
    beepIntervalRef.current = window.setInterval(createBeep, beepInterval);
  };

  // Function to analyze distances and set alert level
  const analyzeDistances = (leftDist: number, rightDist: number) => {
    const minDistance = Math.min(leftDist, rightDist);
    const seuilDanger = 30; // Danger threshold in cm
    const seuilAlerte = 50; // Warning threshold in cm

    if (leftDist < seuilDanger && rightDist < seuilDanger) {
        setAlertLevel('danger');
        setInstruction('⛔ DANGER! STOP IMMEDIATELY');
    } else if (leftDist < seuilDanger) {
        setAlertLevel('warning');
        setInstruction('⚠️ Obstacle on the left! Turn right ⮕');
    } else if (rightDist < seuilDanger) {
        setAlertLevel('warning');
        setInstruction('⚠️ Obstacle on the right! Turn left ⬅');
    } else if (leftDist < seuilAlerte || rightDist < seuilAlerte) {
        setAlertLevel('warning');
        setInstruction('⚠️ Caution! Obstacle nearby.');
    } else {
        setAlertLevel('safe');
        setInstruction('✅ Safe to reverse.');
    }

    // Play beep sound based on the minimum distance
    playBeep(minDistance);
};


  // Function to connect to Arduino via Serial API
  const connectSerial = async () => {
    if (!navigator.serial) {
      setStatus('Web Serial API not supported in this browser');
      return;
    }
    
    try {
      port.current = await navigator.serial.requestPort();
      await port.current.open({ baudRate: 9600 });
      
      setConnected(true);
      setStatus('Connected to Arduino');
      
      const textDecoder = new TextDecoder();
      reader.current = port.current.readable.getReader();
      
      let buffer = "";
      
      // Read data loop
      while (true) {
        try {
          const { value, done } = await reader.current.read();
          if (done) {
            reader.current.releaseLock();
            break;
          }
          
          const text = textDecoder.decode(value);
          buffer += text;
          
          const lines = buffer.split('\n');
          buffer = lines.pop() || "";
          
          for (const line of lines) {
            try {
              const data = JSON.parse(line);
              
              // Update distances
              setLeftDistance(parseFloat(data.gauche.toFixed(2)));
              setRightDistance(parseFloat(data.droite.toFixed(2)));
              
              // Analyze distances for alerts
              analyzeDistances(data.gauche, data.droite);
            } catch (e) {
              console.error("Error parsing JSON:", e);
            }
          }
        } catch (error) {
          console.error("Error reading data:", error);
          break;
        }
      }
    } catch (error) {
      console.error("Error connecting to serial port:", error);
      setStatus('Connection failed');
      setConnected(false);
    }
  };

  // Function to disconnect from Arduino
  const disconnectSerial = async () => {
    if (reader.current) {
      await reader.current.cancel();
      reader.current = null;
    }
    
    if (port.current) {
      await port.current.close();
      port.current = null;
    }
    
    setConnected(false);
    setLeftDistance(null);
    setRightDistance(null);
    setStatus('Disconnected');
    setInstruction('Connect to Arduino to start monitoring');
    setAlertLevel('safe');
    
    // Stop any ongoing beeps
    if (beepIntervalRef.current) {
      clearInterval(beepIntervalRef.current);
      beepIntervalRef.current = null;
    }
  };

  // Function to toggle sound
  const toggleSound = () => {
    setSoundEnabled(!soundEnabled);
    
    if (!soundEnabled && beepIntervalRef.current === null && leftDistance && rightDistance) {
      // Restart beeping if we're turning sound back on
      playBeep(Math.min(leftDistance, rightDistance));
    } else if (soundEnabled && beepIntervalRef.current !== null) {
      // Stop beeping if we're turning sound off
      clearInterval(beepIntervalRef.current);
      beepIntervalRef.current = null;
      
      if (gainNode.current) {
        gainNode.current.gain.value = 0;
      }
    }
  };

  // Function to get color based on distance
  const getColorForDistance = (distance: number | null) => {
    if (distance === null) return 'bg-gray-700';
    if (distance < 30) return 'bg-red-500';
    if (distance < 50) return 'bg-orange-500';
    return 'bg-green-500';
  };

  // Function to calculate progress bar width based on distance
  const getProgressWidth = (distance: number | null) => {
    if (distance === null) return '0%';
    
    // Inverse relationship: closer = more filled
    const percentage = Math.max(0, Math.min(100, 100 - (distance / 2)));
    return `${percentage}%`;
  };

  return (
    <div className="min-h-screen bg-gray-900 text-white flex flex-col">
      {/* Header */}
      <header className="bg-gray-800 p-4 shadow-lg border-b border-blue-500/30">
        <div className="container mx-auto flex justify-between items-center">
          <div className="flex items-center gap-2">
            <Car className="text-blue-400" size={28} />
            <h1 className="text-2xl font-bold bg-clip-text text-transparent bg-gradient-to-r from-blue-400 to-teal-400">
              ReculSafe
            </h1>
          </div>
          <div className="flex items-center gap-4">
            <button 
              onClick={toggleSound} 
              className="p-2 rounded-full hover:bg-gray-700 transition-colors"
              aria-label={soundEnabled ? "Disable sound" : "Enable sound"}
            >
              {soundEnabled ? <Volume2 className="text-blue-400" /> : <Volume className="text-gray-400" />}
            </button>
            <button
              onClick={connected ? disconnectSerial : connectSerial}
              className={`flex items-center gap-2 px-4 py-2 rounded-full transition-colors ${
                connected 
                  ? 'bg-red-500/20 hover:bg-red-500/30 text-red-300' 
                  : 'bg-blue-500/20 hover:bg-blue-500/30 text-blue-300'
              }`}
            >
              {connected ? (
                <>
                  <BluetoothOff size={18} />
                  <span>Disconnect</span>
                </>
              ) : (
                <>
                  <Bluetooth size={18} />
                  <span>Connect Arduino</span>
                </>
              )}
            </button>
          </div>
        </div>
      </header>

      {/* Main content */}
      <main className="flex-1 container mx-auto p-4 md:p-8">
        {/* Status bar */}
        <div className="mb-6 p-4 bg-gray-800/50 backdrop-blur-sm rounded-lg border border-gray-700 flex items-center justify-between">
          <div className="flex items-center gap-2">
            <div className={`w-3 h-3 rounded-full ${connected ? 'bg-green-500' : 'bg-gray-500'}`}></div>
            <span className="text-gray-300">{status}</span>
          </div>
          <div className={`px-4 py-1 rounded-full text-sm font-medium ${
            alertLevel === 'danger' ? 'bg-red-500/20 text-red-300' :
            alertLevel === 'warning' ? 'bg-orange-500/20 text-orange-300' :
            'bg-green-500/20 text-green-300'
          }`}>
            {instruction}
          </div>
        </div>

        {/* Distance displays */}
        <div className="grid grid-cols-1 md:grid-cols-2 gap-6 mb-8">
          {/* Left sensor */}
          <div className="bg-gray-800/50 backdrop-blur-sm rounded-lg border border-gray-700 p-6 relative overflow-hidden">
            <div className="flex justify-between items-start mb-4">
              <h2 className="text-xl font-medium text-gray-300">Left Sensor</h2>
              <div className={`px-3 py-1 rounded-full text-sm ${
                leftDistance === null ? 'bg-gray-700 text-gray-300' :
                leftDistance < 30 ? 'bg-red-500/20 text-red-300' :
                leftDistance < 50 ? 'bg-orange-500/20 text-orange-300' :
                'bg-green-500/20 text-green-300'
              }`}>
                {leftDistance === null ? 'No data' :
                 leftDistance < 30 ? 'Danger' :
                 leftDistance < 50 ? 'Warning' : 'Safe'}
              </div>
            </div>
            
            <div className="flex items-end gap-2 mb-6">
              <div className="text-5xl font-bold">
                {leftDistance !== null ? leftDistance.toFixed(1) : '--'}
              </div>
              <div className="text-xl text-gray-400 mb-1">cm</div>
            </div>
            
            <div className="relative h-4 bg-gray-700 rounded-full overflow-hidden">
              <div 
                className={`absolute top-0 left-0 h-full ${getColorForDistance(leftDistance)} transition-all duration-300 ease-out`}
                style={{ width: getProgressWidth(leftDistance) }}
              ></div>
            </div>
          </div>
          
          {/* Right sensor */}
          <div className="bg-gray-800/50 backdrop-blur-sm rounded-lg border border-gray-700 p-6 relative overflow-hidden">
            <div className="flex justify-between items-start mb-4">
              <h2 className="text-xl font-medium text-gray-300">Right Sensor</h2>
              <div className={`px-3 py-1 rounded-full text-sm ${
                rightDistance === null ? 'bg-gray-700 text-gray-300' :
                rightDistance < 30 ? 'bg-red-500/20 text-red-300' :
                rightDistance < 50 ? 'bg-orange-500/20 text-orange-300' :
                'bg-green-500/20 text-green-300'
              }`}>
                {rightDistance === null ? 'No data' :
                 rightDistance < 30 ? 'Danger' :
                 rightDistance < 50 ? 'Warning' : 'Safe'}
              </div>
            </div>
            
            <div className="flex items-end gap-2 mb-6">
              <div className="text-5xl font-bold">
                {rightDistance !== null ? rightDistance.toFixed(1) : '--'}
              </div>
              <div className="text-xl text-gray-400 mb-1">cm</div>
            </div>
            
            <div className="relative h-4 bg-gray-700 rounded-full overflow-hidden">
              <div 
                className={`absolute top-0 left-0 h-full ${getColorForDistance(rightDistance)} transition-all duration-300 ease-out`}
                style={{ width: getProgressWidth(rightDistance) }}
              ></div>
            </div>
          </div>
        </div>

        {/* Visual representation */}
        <div className="bg-gray-800/50 backdrop-blur-sm rounded-lg border border-gray-700 p-6">
          <h2 className="text-xl font-medium text-gray-300 mb-6">Obstacle Visualization</h2>
          
          <div className="relative w-full max-w-2xl mx-auto h-64 bg-gray-800 rounded-lg border border-gray-700 overflow-hidden">
            {/* Car representation */}
            <div className="absolute top-1/2 left-1/2 transform -translate-x-1/2 -translate-y-1/2">
              <Car size={64} className="text-blue-400" />
            </div>
            
            {/* Left sensor zone */}
            <div className="absolute top-0 left-0 w-1/2 h-full flex items-center justify-start pl-4">
              {leftDistance !== null && leftDistance < 100 && (
                <div 
                  className={`h-16 rounded-l-full ${
                    leftDistance < 30 ? 'bg-red-500/30' :
                    leftDistance < 50 ? 'bg-orange-500/30' :
                    'bg-green-500/30'
                  } flex items-center justify-end pr-2 transition-all duration-300`}
                  style={{ width: `${Math.min(100, 100 - leftDistance)}%` }}
                >
                  {leftDistance < 50 && (
                    <AlertTriangle 
                      size={24} 
                      className={leftDistance < 30 ? 'text-red-400' : 'text-orange-400'} 
                    />
                  )}
                </div>
              )}
            </div>
            
            {/* Right sensor zone */}
            <div className="absolute top-0 right-0 w-1/2 h-full flex items-center justify-end pr-4">
              {rightDistance !== null && rightDistance < 100 && (
                <div 
                  className={`h-16 rounded-r-full ${
                    rightDistance < 30 ? 'bg-red-500/30' :
                    rightDistance < 50 ? 'bg-orange-500/30' :
                    'bg-green-500/30'
                  } flex items-center justify-start pl-2 transition-all duration-300`}
                  style={{ width: `${Math.min(100, 100 - rightDistance)}%` }}
                >
                  {rightDistance < 50 && (
                    <AlertTriangle 
                      size={24} 
                      className={rightDistance < 30 ? 'text-red-400' : 'text-orange-400'} 
                    />
                  )}
                </div>
              )}
            </div>
            
            {/* Distance markers */}
            <div className="absolute bottom-2 left-0 w-full flex justify-center text-xs text-gray-500">
              <div className="w-full max-w-xs flex justify-between">
                <span>100cm</span>
                <span>50cm</span>
                <span>0cm</span>
              </div>
            </div>
          </div>
        </div>
      </main>

      {/* Footer */}
      <footer className="bg-gray-800 p-4 border-t border-blue-500/30">
        <div className="container mx-auto text-center text-gray-500 text-sm">
          ReculSafe © 2025 - Smart Reverse Radar System
        </div>
      </footer>
    </div>
  );
}

export default App;